#include <argp.h>
#include <arpa/inet.h>
#include <errno.h>
#include <error.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/stat.h>
#include <sys/socket.h>
#include <netdb.h>
#include <net/if.h>
#include <netinet/in.h>
#include <unistd.h>
#include <sys/utsname.h>

#include <sys/param.h>
#include <sys/ioctl.h>
#include <net/if.h>

#define MAX_NUM_IFREQ 64
#define MAX_IP 64

typedef struct eth_info {
	int	up;
	int	loop;
	int	connect;
	char	name[MAX_IP];
	char	mac[MAX_IP];
	char	addr[MAX_IP];
} eth_info_t;

#include "mct_test.h"

extern int verbose;
extern char dev_file_path[1024];


#define PATH_PROCNET_DEV "/proc/net/dev"

/* One per interface mentioned on the command line.  */
struct ifconfig {
	char *name;
};

/* Array of all interfaces on the command line.  */
static struct ifconfig *ifs;

/* Size of IFS.  */
static int nifs = 0;

static struct ifconfig ifconfig_initializer = {
	NULL	/* name */
};

static int cmp_if_name (const void *a, const void *b) {
	const struct ifconfig *ifa = a;
	const struct ifconfig *ifb = b;
	return strcmp (ifa->name, ifb->name);
}

static char *fread_file (FILE *stream, size_t *length) {
	char *buf = NULL;
	size_t alloc = BUFSIZ;
	struct stat st;
	size_t size = 0; /* number of bytes read so far */
	int save_errno;

	/* For a regular file, allocate a buffer that has exactly the right
	   size.  This avoids the need to do dynamic reallocations later.  */
	if (fstat (fileno (stream), &st) >= 0 && S_ISREG (st.st_mode)) {
		off_t pos = ftello (stream);

		if (pos >= 0 && pos < st.st_size) {
			off_t alloc_off = st.st_size - pos;

			/* '1' below, accounts for the trailing NUL.  */
			if ((int)(SIZE_MAX - 1) < alloc_off) {
				errno = ENOMEM;
				return NULL;
			}

			alloc = alloc_off + 1;
		}
	}

	if (!(buf = malloc (alloc))) return NULL; /* errno is ENOMEM.  */

	for (;;) {
		/* This reads 1 more than the size of a regular file
		   so that we get eof immediately.  */
		char *new_buf;
		size_t requested = alloc - size;
		size_t count = fread (buf + size, 1, requested, stream);
		size += count;

		if (count != requested) {
			save_errno = errno;
			if (ferror (stream)) break;

			/* Shrink the allocated memory if possible.  */
			if (size < alloc - 1) {
				char *smaller_buf = realloc (buf, size + 1);
				if (smaller_buf != NULL)
					buf = smaller_buf;
			}

			buf[size] = '\0';
			*length = size;
			return buf;
		}

		if (alloc == SIZE_MAX) {
			save_errno = ENOMEM;
			break;
		}

		if (alloc < SIZE_MAX - alloc / 2)
			alloc = alloc + alloc / 2;
		else
			alloc = SIZE_MAX;

		if (!(new_buf = realloc (buf, alloc))) {
			save_errno = errno;
			break;
		}

		buf = new_buf;
	}

	free (buf);
	errno = save_errno;
	return NULL;
}


static char *read_file (const char *filename, size_t *length, const char *mode) {
	FILE *stream = fopen (filename, mode);
	char *out;
	int save_errno;

	if (!stream) return NULL;

	out = fread_file (stream, length);
	save_errno = errno;

	if (fclose (stream) != 0) {
		if (out) {
			save_errno = errno;
			free (out);
		}
		errno = save_errno;
		return NULL;
	}

	return out;
}


static struct if_nameindex *linux_if_nameindex (void) {
	char *content, *it;
	size_t length = 0, index, n = 0;
	struct if_nameindex *idx = NULL;

	content = read_file (PATH_PROCNET_DEV, &length, "r");
	if (content == NULL) return NULL;

	/* Count how many interfaces we have.  */
	it = content;
	do {
		it = memchr (it + 1, ':', length - (it - content));
		n++;
	} while (it);

	idx = malloc (n * sizeof (*idx));
	if (idx == NULL) {
		int saved_errno = errno;
		free (content);
		errno = saved_errno;
		return NULL;
	}

	for (it = memchr (content, ':', length), index = 0; it;
	     it = memchr (it, ':', length - (it - content)), index++)
	{
		char *start = it - 1;
		*it = '\0';

		while (*start != ' ' && *start != '\n')
			start--;

		idx[index].if_name = strdup (start + 1);
		idx[index].if_index = index + 1;

		if (idx[index].if_name == NULL) {
			int saved_errno = errno;
			free (content);
			errno = saved_errno;
			return NULL;
		}
	}

	idx[index].if_index = 0;
	idx[index].if_name = NULL;

	free (content);
	return idx;
}



static int get_all_iface(void) {
	struct ifconfig *ifp = ifs;
	struct if_nameindex *ifnx, *ifnxp;

	ifnx = ifnxp = linux_if_nameindex();
	if (!ifnx) {
		myprintf("could not get list of interfaces\n");
		return -1;
	}
	while (ifnxp->if_index != 0 || ifnxp->if_name != NULL)
	{
		ifs = realloc (ifs, ++nifs * sizeof (struct ifconfig));
		if (!ifs) {
			myprintf("can't get memory for interface configuration\n");
			free(ifs); //???
			return -2;
		}
		ifp = &ifs[nifs - 1];
		*ifp = ifconfig_initializer;
		ifp->name = ifnxp->if_name;
		ifnxp++;
	}

	qsort (ifs, nifs, sizeof (ifs[0]), cmp_if_name);
	free(ifnx); //???
	return 0;
}

static void fill_iface_info(int fd, int i, struct ifreq *ptr, eth_info_t *e) {
	unsigned char mac[20]={0};
	struct sockaddr_in addrtmp;

	myprintf("---===<<<[%d]>>>===---\n", i);

        myprintf("[%d] Name = %s\n", i, ptr->ifr_name);
	snprintf(e[i].name, 64, "%s", ptr->ifr_name);

        if ( ioctl(fd,SIOCGIFHWADDR, ptr) >= 0) 	{
		memcpy(mac,(unsigned char *)&(ptr->ifr_hwaddr.sa_data),sizeof(struct sockaddr));
		myprintf("[%d] MAC = %02X:%02X:%02X:%02X:%02X:%02X\n",
			  i,mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
		snprintf(e[i].mac, 64, "%02X:%02X:%02X:%02X:%02X:%02X",
			 mac[0],mac[1],mac[2],mac[3],mac[4],mac[5]);
	}

        if ( ioctl(fd, SIOCGIFADDR, ptr) >= 0) {
		memcpy(&addrtmp,&(ptr->ifr_addr),sizeof(addrtmp));
		myprintf("[%d] Address = %s\n",i, inet_ntoa(addrtmp.sin_addr));
		snprintf(e[i].addr, 64, "%s", inet_ntoa(addrtmp.sin_addr));
	}

        if ( ioctl(fd, SIOCGIFBRDADDR, ptr) >= 0) {
		memcpy(&addrtmp,&(ptr->ifr_broadaddr),sizeof(addrtmp));
		myprintf("[%d] BroadAddress = %s\n",i, inet_ntoa(addrtmp.sin_addr));
	}

        if ( ioctl(fd, SIOCGIFNETMASK, ptr) >= 0) {
		memcpy(&addrtmp,&(ptr->ifr_addr),sizeof(addrtmp));
		myprintf("[%d] MaskAddress = %s\n",i, inet_ntoa(addrtmp.sin_addr));
	}

        if ( ioctl(fd, SIOCGIFFLAGS, ptr) >= 0) {
		myprintf("[%d] IFF_UP=%s\n",i, ptr->ifr_flags&0x1?"ON":"OFF");
		e[i].up = (ptr->ifr_flags & 0x1) ? 1 : 0;
		/*
		myprintf("[%d] IFF_BROADCAST=%s\n",i, ptr->ifr_flags&0x2?"ON":"OFF");
		myprintf("[%d] IFF_DEBUG=%s\n",i, ptr->ifr_flags&0x4?"ON":"OFF");
		myprintf("[%d] IFF_LOOPBACK=%s\n",i, ptr->ifr_flags&0x8?"ON":"OFF");
		*/
		e[i].loop = (ptr->ifr_flags & 0x8) ? 1 : 0;
		myprintf("[%d] IFF_RUNNING=%s\n",i, ptr->ifr_flags&0x40?"ON":"OFF");
		e[i].connect = (ptr->ifr_flags & 0x40) ? 1 : 0;
		/*
		myprintf("[%d] IFF_P2P=%s\n",i, ptr->ifr_flags&0x10?"ON":"OFF");
		myprintf("[%d] IFF_NOTRAILERS=%s\n",i, ptr->ifr_flags&0x20?"ON":"OFF");
		myprintf("[%d] IFF_NOARP=%s\n",i, ptr->ifr_flags&0x80?"ON":"OFF");
		*/
	}
}


static int fill_info_about_all_eth(int sfd, eth_info_t *e) {
	int count = 0;
	struct ifconfig *ifp;
	struct ifreq ifr;

	if (get_all_iface()) {
		myprintf("get all iface error\n");
		return -1;
	}

	for (ifp = ifs; ifp < ifs + nifs; ifp++) {
		memset (&ifr, 0, sizeof (ifr));
		strncpy (ifr.ifr_name, (char *)ifp->name, IFNAMSIZ);
		ifr.ifr_name[IFNAMSIZ - 1] = '\0';
		fill_iface_info(sfd, count, &ifr, e);
		count++;
	}

	return count;
}

/**
 * \return 0 - if error
 * \return eth i-face numbers include lo
 */
static int fill_info_about_up_eth(int fd, eth_info_t *e) {
	struct ifconf       ifcfg;
	struct ifreq        buf[MAX_NUM_IFREQ];
	struct ifreq        *ptr;
	int num_ifreq;
	int i;

	ifcfg.ifc_len = sizeof(buf);
	ifcfg.ifc_buf = (char *) buf;

	if (ioctl(fd, SIOCGIFCONF, &ifcfg) < 0) {
		myprintf("ioctl SIOCGIFCONF: %s\n", strerror(errno));
		return 0;
	}

	num_ifreq = ifcfg.ifc_len / sizeof(struct ifreq);
	if (num_ifreq > MAX_NUM_IFREQ)
		num_ifreq = MAX_NUM_IFREQ;
	myprintf("::: {%d} :::\n", num_ifreq);
	for (ptr = ifcfg.ifc_req, i = 0 ; i < num_ifreq; ptr++, i++) {
		fill_iface_info(fd, i, ptr, e);
	}

	return num_ifreq;
}


int fill_info_about_eth(int fd, eth_info_t *e) {
	int count = fill_info_about_all_eth(fd, e);
	if (count <= 0) {
		myprintf("Old way --------------\n");
		count = fill_info_about_up_eth(fd, e);
	}
	return count;
}



int run_eth_test(void) {
	int i = 0, res = TEST_FAIL;
	eth_info_t eth[MAX_NUM_IFREQ];
	int eth_num = 0; /* ethernet numbers include loopback */
	int32_t tmpsocket;

	myprintf("run ethernet test for:  %s\n", dev_file_path);

	tmpsocket = socket(AF_INET, SOCK_STREAM, 0);
	if (tmpsocket >= 0) {
		eth_num = fill_info_about_eth(tmpsocket, eth);
		res = TEST_SKIP;
	} else {
		myprintf("Error: %s (%d)\n",
			 strerror(errno), errno);
	}
	close(tmpsocket);

	myprintf("Check dev: [%s]\n", dev_file_path);
	for (i = 0; i < eth_num; i++) {
		if (!strcmp(eth[i].name, dev_file_path)) {
			if (eth[i].up && eth[i].connect) {
				res = TEST_PASS;
				break;
			}
		}
	}

	return res;
}
