#ifndef _MC404_H
#	define _MC404_H

#undef PRINTK
#ifdef MC404_DEBUG
#  ifdef __KERNEL__
     /* This one if debugging is on, and kernel space */
#    define PRINTK(fmt, args...) printk( KERN_DEBUG fmt, ## args)
#  else
     /* This one for user space */
#    define PRINTK(fmt, args...) fprintf(stderr, fmt, ## args)
#  endif /* #ifdef __KERNEL__ */
#else
#  define PRINTK(fmt, args...) /* not debugging: nothing */
#endif /* #ifdef MC404_DEBUG */

#ifndef SHR_WORD_OFFSET
#	define SHR_WORD_OFFSET(x) ((x) & ~0xC0000000)
#endif
#ifndef SHR_BYTE_OFFSET
#	define SHR_BYTE_OFFSET(x) (((x) & ~0xC0000000) << 2)
#endif

#ifndef swab32
#define swab32(x) ((x>>24) | ((x>>8) & 0xff00) | ((x<<8) & 0xff0000) | ((x<<24) & 0xff000000))
#endif

#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

#define MAX_DEV_NUM 10
//#define MC404_DEF_TIMEOUT (1*HZ) /* Default timeout 1000 mS */
#define MC404_DEF_TIMEOUT (3*HZ) /* Default timeout 1000 mS */
#define MC404_MEM_SIZE 0x8000000 /* Memory size in bytes */
#define FNAME_LEN 8		/* Device file name length */
#define BOOT_MAGIC 0x25091954

/* LED switch */
#define LED_ON		1
#define LED_OFF		0
#define LED_TOGGLE	2

/* Local configuration registers of 9054 (offset in bytes) */
#define LAS0RR  0x00
#define LAS0BA  0x04
#define MARBR   0x08
#define BIGEND  0x0C
#define EROMRR  0x10
#define EROMBA  0x14
#define LBRD0   0x18
#define DMRR    0x1C
#define DMLBAM  0x20
#define DMLBAI  0x24
#define DMPBAM  0x28
#define DMCFGA  0x2C
#define LAS1RR  0xF0
#define LAS1BA  0xF4
#define LBRD1   0xF8
#define DMDAC   0xFC

/* RUNTIME registers of 9054 (offset in bytes) */
#define MBOX0    0x40
#define MBOX1    0x44
#define MBOX2    0x48
#define MBOX3    0x4C
#define MBOX4    0x50
#define MBOX5    0x54
#define MBOX6    0x58
#define MBOX7    0x5C
#define P2LDBELL 0x60
#define L2PDBELL 0x64
#define INTCSR   0x68
#define CNTRL    0x6C
#define PCIHIDR  0x70
#define PCIHREV  0x74
#define DMAMODE0 0x80
#define DMAMODE1 0x94
#define DMAPADR0 0x84
#define DMAPADR1 0x98
#define DMALADR0 0x88
#define DMALADR1 0x9C
#define DMASIZ0  0x8C
#define DMASIZ1  0xA0
#define DMADPR0  0x90
#define DMADPR1  0xA4
#define DMACSR0  0xA8
#define DMACSR1  0xA9
#define DMATHR   0xB0
#define DMADAC0  0xB4
#define DMADAC1  0xB8

#define DMA_LOCAL_MASK0 0x10000000 /* Local PLX address mask (bank 0) */
#define DMA_LOCAL_MASK1 0x18000000 /* Local PLX address mask (bank 1) */
//#define DMA_MAX_PAGES   6144	   /* Max. pages number in DMA mode */

/* DMA MODE bits */
#define DMA_LOCAL_WIDTH8 0
#define DMA_LOCAL_WIDTH16 1
#define DMA_LOCAL_WIDTH32 3
#define DMA_LOCAL_BURST_ENABLE (1 << 8)
#define DMA_SG_MODE (1 << 9)
#define DMA_INT_ENABLE (1 << 10)
#define DMA_LOC_ADDR_INC (1 << 11)
#define DMA_CHANNEL_INT_PCI (1 << 17)
#define DMA_DAC_CHAIN_LOAD (1 << 18)

/* DMA DPR bits */
#define DMA_DPR_PCI 1
#define DMA_DPR_LOCAL 0
#define DMA_DPR_END_OF_CHAIN 2
#define DMA_DPR_TERMINAL_COUNT_INTR 4
#define DMA_PCI_TO_LOCAL 0
#define DMA_LOCAL_TO_PCI 8

/* DMA CSR bits */
#define DMA_CHANNEL_ENABLE 1
#define DMA_CHANNEL_START 2
#define DMA_CHANNEL_ABORT 4
#define DMA_CLEAR_INTERRUPT 8
#define DMA_CHANNEL_DONE 0x10

/* INTCSR bits */
#define INTCSR_DMA0_ENABLE    		(1 << 18)
#define INTCSR_DMA1_ENABLE    		(1 << 19)
#define INTCSR_DMA0_ACTIVE    		(1 << 21)
#define INTCSR_DMA1_ACTIVE    		(1 << 22)
#define INTCSR_LOC_INT_ENABLE 		(1 << 11)
#define INTCSR_MAILBOX_INT_ENABLE	(1 << 3)
#define INTCSR_PCI_INT_ENABLE		(1 << 8)
#define INTCSR_PCI_DOORBELL_INT_ENABLE	(1 << 9)
#define INTCSR_PCI_DOORBELL_INT_ACTIVE	(1 << 13)
#define INTCSR_LOC_INPUT_INT_ACTIVE	(1 << 15)
#define INTCSR_LOC_INT_OUTPUT_ENABLE	(1 << 16)
#define INTCSR_LOC_DOORBELL_INT_ENABLE	(1 << 17)

#include <linux/ioctl.h>
#define MC404_IOCTL_MAGIC 'k'

/*----------------------------*/
/* IOCTL requests             */
/*----------------------------*/
/* Get number of devices */
#define IOCTL_MC404_GET_DEV_NUM			_IOW(MC404_IOCTL_MAGIC, 40, long)
/* Get boot test result */
#define IOCTL_MC404_GET_BOOT_RC			_IOW(MC404_IOCTL_MAGIC, 41, long)
/* Set timeout in mS (default 1000 mS) */
#define IOCTL_MC404_SET_TIMEOUT			_IOR(MC404_IOCTL_MAGIC, 13, long)
/* Get current timeout value in mS */
#define IOCTL_MC404_GET_TIMEOUT			_IOW(MC404_IOCTL_MAGIC, 14, long)
/* Enable interrupts local->PCI */
#define IOCTL_MC404_INT_ENABLE			_IO(MC404_IOCTL_MAGIC, 15)
/* Disable interrupts local->PCI */
#define IOCTL_MC404_INT_DISABLE			_IO(MC404_IOCTL_MAGIC, 16)

/* Enable doorbell interrupt from PCI to local */
#define IOCTL_MC404_PDB_INT_ENABLE		_IO(MC404_IOCTL_MAGIC, 34)
/* Disable doorbell interrupt from PCI to local */
#define IOCTL_MC404_PDB_INT_DISABLE		_IO(MC404_IOCTL_MAGIC, 35)
/* Enable doorbell interrupt from local to PCI */
#define IOCTL_MC404_LDB_INT_ENABLE		_IO(MC404_IOCTL_MAGIC, 36)
/* Disable doorbell interrupt from local to PCI */
#define IOCTL_MC404_LDB_INT_DISABLE		_IO(MC404_IOCTL_MAGIC, 37)
/* Enable mailbox interrupt from PCI */
#define IOCTL_MC404_MBOX_INT_ENABLE		_IO(MC404_IOCTL_MAGIC, 38)
/* Disable mailbox interrupt from PCI */
#define IOCTL_MC404_MBOX_INT_DISABLE		_IO(MC404_IOCTL_MAGIC, 39)

/* Enable interrupts from 6403 chips */
#define IOCTL_MC404_CHIP_ENABLE			_IOR(MC404_IOCTL_MAGIC, 17, long)
/* Disable interrupts from 6403 chips */
#define IOCTL_MC404_CHIP_DISABLE		_IOR(MC404_IOCTL_MAGIC, 18, long)
/* Set interrupt mask */
#define IOCTL_MC404_SET_MASK			_IOR(MC404_IOCTL_MAGIC, 19, long)
/* Get interrupt mask */
#define IOCTL_MC404_GET_MASK			_IOW(MC404_IOCTL_MAGIC, 33, long)
/* Wait for interrupt from MC4.04.
After performance of this request it's nesessary receive the interrupt status
(IOCTL_MC404_GET_INTERRUPT_STATUS). If status is 0, then timeout.
Besides this request allows the following interrupts*/
#define IOCTL_MC404_WAIT_INTERRUPT		_IO(MC404_IOCTL_MAGIC, 20)
/* Cancel interrupt expectation */
#define IOCTL_MC404_CANCEL_WAIT			_IO(MC404_IOCTL_MAGIC, 21)
/* Get interrupt status (see IOCTL_MC404_WAIT_INTERRUPT) */
#define IOCTL_MC404_GET_INTERRUPT_STATUS	_IOW(MC404_IOCTL_MAGIC, 22, long)
/* Write 32-bit word to the PLX register.
Parameter of this request is the pointer to structure mc_port (see below )*/
#define IOCTL_MC404_PORT_WRITE			_IOR(MC404_IOCTL_MAGIC, 23, long)
/* Read 32-bit word from the PLX register.
Parameter of this request is the pointer to structure mc_port (see below )*/
#define IOCTL_MC404_PORT_READ			_IOR(MC404_IOCTL_MAGIC, 24, long)
/* Reset MC4.04 */
#define IOCTL_MC404_RESET			_IO(MC404_IOCTL_MAGIC, 25)
/* Interrupt MC4.04 */
#define IOCTL_MC404_INTERRUPT			_IOR(MC404_IOCTL_MAGIC, 26, long)
/* Turn on the red LED */
#define IOCTL_MC404_LED_ON			_IO(MC404_IOCTL_MAGIC, 27)
/* Turn off the red LED */
#define IOCTL_MC404_LED_OFF			_IO(MC404_IOCTL_MAGIC, 28)
/* Toggle the red LED */
#define IOCTL_MC404_LED_TOGGLE			_IO(MC404_IOCTL_MAGIC, 29)
#define IOCTL_MC404_GET_INT_COUNT		_IOW(MC404_IOCTL_MAGIC, 30, long)
#define IOCTL_MC404_RESET_INT_COUNT		_IO(MC404_IOCTL_MAGIC, 31)

#ifdef __KERNEL__
#include <linux/spinlock.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/cdev.h>

#define RC_MODULE_VENDOR_ID 0x6403
#define MC404_DEVICE_ID    0x334
#define MC404_MAJOR        249
#define MC404_FILENAME     "mc404"
#define SUCCESS 0

/* Internal structure of parameters */
struct mc404_device
{
	int		num;
	char		*name;
	int		open;		/* open flag */

	unsigned long	mem0;		/* not mapped */
	unsigned long	mem1;		/* not mapped */
	unsigned long	mmio;		/* not mapped */
	unsigned long	mmio_m;		/* mapped registers */
	unsigned long	space0;		/* mapped shared memory */
	unsigned long	space1;		/* mapped space1 */
	unsigned long	size_mmio;	/* size of mmio */
	unsigned long	size_space0;	/* size of space0 */
	unsigned long	size_space1;	/* size of space1 */

	int		irq;		/* IRQ */
	volatile int	int_reg;	/* Duplicate of the interrupt register */
	volatile unsigned long	interrupt;	/* Flag of interrupt processing */
	int		timeout;	/* Timeout */
	struct cdev	cdev;
	wait_queue_head_t queue;
	spinlock_t	lock;		/* spinlock */
	struct semaphore sem;
	volatile int int_dma;
	struct page **pages;
//	unsigned int npages;
	struct scatterlist *sclist;
	struct plx_scatterlist *plx_sclist;
	int boot_rc;			/* result of boot test */
	int nirq;			/* irq counter */
};

struct plx_scatterlist
{
	u32 pci_addr;
	u32 local_addr;
	u32 size;
	u32 next;
};

#define TYPE(dev)	(MINOR(dev) >> 4)
#define NUM(dev)	(MINOR(dev) & 0xF)
#endif /* #ifdef __KERNEL__ */

/* Structure for read/write PLX registers */
struct mc_port
{
	unsigned int addr;	/* Register offset */
	unsigned int value;	/* Value for read/write */
};

#endif /* #ifndef _MC404_H */
