#ifndef _MC7603_H
#	define _MC7603_H

#ifdef ENABLE_RT
    #ifdef E90
	#ifdef CONFIG_PREEMPT_RT
	    #define RT_MODE
	#endif
    #endif
#endif

#undef PRINTK
#ifdef MC7603_DEBUG
#  ifdef __KERNEL__
     /* This one if debugging is on, and kernel space */
#    define PRINTK(fmt, args...) printk( KERN_DEBUG fmt, ## args)
#  else
     /* This one for user space */
#    define PRINTK(fmt, args...) fprintf(stderr, fmt, ## args)
#  endif /* #ifdef __KERNEL__ */
#else
#  define PRINTK(fmt, args...) /* not debugging: nothing */
#endif /* #ifdef MC7603_DEBUG */

#ifndef SHR_WORD_OFFSET
#	define SHR_WORD_OFFSET(x) ((x) & ~0xC0000000)
#endif
#ifndef SHR_BYTE_OFFSET
#	define SHR_BYTE_OFFSET(x) (((x) & ~0xC0000000) << 2)
#endif

#ifndef swab32
#define swab32(x) ((x>>24) | ((x>>8) & 0xff00) | ((x<<8) & 0xff0000) | ((x<<24) & 0xff000000))
#endif

#ifdef RT_MODE
#	define SPIN_LOCK(a) raw_spin_lock(a)
#	define SPIN_LOCK_IRQ(a) raw_spin_lock_irq(a)
#	define SPIN_LOCK_IRQSAVE(a, b) raw_spin_lock_irqsave(a, b)
#	define SPIN_LOCK_BH(a) raw_spin_lock_bh(a)
#	define SPIN_UNLOCK(a) raw_spin_unlock(a)
#	define SPIN_UNLOCK_IRQ(a) raw_spin_unlock_irq(a)
#	define SPIN_UNLOCK_IRQRESTORE(a, b) raw_spin_unlock_irqrestore(a, b)
#	define SPIN_UNLOCK_BH(a) raw_spin_unlock_bh(a)
#	define SPIN_LOCK_INIT(a) raw_spin_lock_init(a)
#else
#	define SPIN_LOCK(a) spin_lock(a)
#	define SPIN_LOCK_IRQ(a) spin_lock_irq(a)
#	define SPIN_LOCK_IRQSAVE(a, b) spin_lock_irqsave(a, b)
#	define SPIN_LOCK_BH(a) spin_lock_bh(a)
#	define SPIN_UNLOCK(a) spin_lock(a)
#	define SPIN_UNLOCK_IRQ(a) spin_unlock_irq(a)
#	define SPIN_UNLOCK_IRQRESTORE(a, b) spin_unlock_irqrestore(a, b)
#	define SPIN_UNLOCK_BH(a) spin_unlock_bh(a)
#	define SPIN_LOCK_INIT(a) spin_lock_init(a)
#endif

#define SEEK_SET 0
#define SEEK_CUR 1
#define SEEK_END 2

#define MAX_DEV_NUM 10
#define MC7603_DEF_TIMEOUT (1*HZ) /* Default timeout 1000 mS */
#define MC7603_MEM_SIZE 0x8000000 /* Memory size in bytes */
#define FNAME_LEN 8		/* Device file name length */
#define BOOT_MAGIC 0x25091954

/* LED switch */
#define LED_ON		1
#define LED_OFF		0
#define LED_TOGGLE	2

/* Local configuration registers of 9054 (offset in bytes) */
#define LAS0RR  0x00
#define LAS0BA  0x04
#define MARBR   0x08
#define BIGEND  0x0C
#define EROMRR  0x10
#define EROMBA  0x14
#define LBRD0   0x18
#define DMRR    0x1C
#define DMLBAM  0x20
#define DMLBAI  0x24
#define DMPBAM  0x28
#define DMCFGA  0x2C
#define LAS1RR  0xF0
#define LAS1BA  0xF4
#define LBRD1   0xF8
#define DMDAC   0xFC

/* RUNTIME registers of 9054 (offset in bytes) */
#define MBOX0    0x40
#define MBOX1    0x44
#define MBOX2    0x48
#define MBOX3    0x4C
#define MBOX4    0x50
#define MBOX5    0x54
#define MBOX6    0x58
#define MBOX7    0x5C
#define P2LDBELL 0x60
#define L2PDBELL 0x64
#define INTCSR   0x68
#define CNTRL    0x6C
#define PCIHIDR  0x70
#define PCIHREV  0x74
#define DMAMODE0 0x80
#define DMAMODE1 0x94
#define DMAPADR0 0x84
#define DMAPADR1 0x98
#define DMALADR0 0x88
#define DMALADR1 0x9C
#define DMASIZ0  0x8C
#define DMASIZ1  0xA0
#define DMADPR0  0x90
#define DMADPR1  0xA4
#define DMACSR0  0xA8
#define DMACSR1  0xA9
#define DMATHR   0xB0
#define DMADAC0  0xB4
#define DMADAC1  0xB8

#define DMA_LOCAL_MASK0 0x10000000 /* Local PLX address mask (bank 0) */
#define DMA_LOCAL_MASK1 0x18000000 /* Local PLX address mask (bank 1) */
//#define DMA_MAX_PAGES   6144	   /* Max. pages number in DMA mode */

/* DMA MODE bits */
#define DMA_LOCAL_WIDTH8 0
#define DMA_LOCAL_WIDTH16 1
#define DMA_LOCAL_WIDTH32 3
#define DMA_LOCAL_BURST_ENABLE (1 << 8)
#define DMA_SG_MODE (1 << 9)
#define DMA_INT_ENABLE (1 << 10)
#define DMA_LOC_ADDR_INC (1 << 11)
#define DMA_CHANNEL_INT_PCI (1 << 17)
#define DMA_DAC_CHAIN_LOAD (1 << 18)

/* DMA DPR bits */
#define DMA_DPR_PCI 1
#define DMA_DPR_LOCAL 0
#define DMA_DPR_END_OF_CHAIN 2
#define DMA_DPR_TERMINAL_COUNT_INTR 4
#define DMA_PCI_TO_LOCAL 0
#define DMA_LOCAL_TO_PCI 8

/* DMA CSR bits */
#define DMA_CHANNEL_ENABLE 1
#define DMA_CHANNEL_START 2
#define DMA_CHANNEL_ABORT 4
#define DMA_CLEAR_INTERRUPT 8
#define DMA_CHANNEL_DONE 0x10

/* INTCSR bits */
#define INTCSR_DMA0_ENABLE    		(1 << 18)
#define INTCSR_DMA1_ENABLE    		(1 << 19)
#define INTCSR_DMA0_ACTIVE    		(1 << 21)
#define INTCSR_DMA1_ACTIVE    		(1 << 22)
#define INTCSR_LOC_INT_ENABLE 		(1 << 11)
#define INTCSR_MAILBOX_INT_ENABLE	(1 << 3)
#define INTCSR_PCI_INT_ENABLE		(1 << 8)
#define INTCSR_PCI_DOORBELL_INT_ENABLE	(1 << 9)
#define INTCSR_PCI_DOORBELL_INT_ACTIVE	(1 << 13)
#define INTCSR_LOC_INPUT_INT_ACTIVE	(1 << 15)
#define INTCSR_LOC_INT_OUTPUT_ENABLE	(1 << 16)
#define INTCSR_LOC_DOORBELL_INT_ENABLE	(1 << 17)

/* MC76.03 register index */
enum ruk { MC_INTR, MC_INTMASK, MC_RESET, MC_INFO,		// 0, 1, 2, 3,		| 0, 1, 2, 3,
MC_ROM1_ADDR, MC_ROM1_SIZE, MC_ROM1_BUFFER, MC_ROM1_COMMAND,	// 4, 5, 6, 7,		| 4, 5, 6, 7,
MC_RUK8, MC_RUK9, MC_RUK10, MC_RUK11,				// 8, 9, A, B		| 8, 9, 10, 11,
MC_RUK12, MC_RUK13, MC_RUK14, MC_RUK15,				// C, D, E, F,		| 12, 13, 14, 15,
CPU0MB0, CPU0MB1, CPU0MB2, CPU0MB3,				// 10, 11, 12, 13,	| 16, 17, 18, 19,
CPU1MB0, CPU1MB1, CPU1MB2, CPU1MB3,				// 14, 15, 16, 17,	| 20, 21, 22, 23,
CPU2MB0, CPU2MB1, CPU2MB2, CPU2MB3,				// 18, 19, 1A, 1B,	| 24, 25, 26, 27,
CPU3MB0, CPU3MB1, CPU3MB2, CPU3MB3,				// 1C, 1D, 1E, 1F,	| 28, 29, 30, 31,
MC_RUK32, MC_RUK33, MC_RUK34, MC_RUK35,				// 20, 21, 22, 23,	| 32, 33, 34, 35,
MC_ROM2_ADDR, MC_ROM2_SIZE, MC_ROM2_BUFFER, MC_ROM2_COMMAND};	// 24, 25, 26, 27	| 36, 37, 38, 39

#include <linux/ioctl.h>
#define MC7603_IOCTL_MAGIC 'j'

/*----------------------------*/
/* IOCTL requests             */
/*----------------------------*/
/* Get number of devices */
#define IOCTL_MC7603_GET_DEV_NUM			_IOW(MC7603_IOCTL_MAGIC, 40, long)
/* Get boot test result */
#define IOCTL_MC7603_GET_BOOT_RC			_IOW(MC7603_IOCTL_MAGIC, 41, long)
/* Set timeout in mS (default 1000 mS) */
#define IOCTL_MC7603_SET_TIMEOUT			_IOR(MC7603_IOCTL_MAGIC, 13, long)
/* Get current timeout value in mS */
#define IOCTL_MC7603_GET_TIMEOUT			_IOW(MC7603_IOCTL_MAGIC, 14, long)
/* Enable interrupts local->PCI */
#define IOCTL_MC7603_INT_ENABLE			_IO(MC7603_IOCTL_MAGIC, 15)
/* Disable interrupts local->PCI */
#define IOCTL_MC7603_INT_DISABLE			_IO(MC7603_IOCTL_MAGIC, 16)

/* Enable doorbell interrupt from PCI to local */
#define IOCTL_MC7603_PDB_INT_ENABLE		_IO(MC7603_IOCTL_MAGIC, 34)
/* Disable doorbell interrupt from PCI to local */
#define IOCTL_MC7603_PDB_INT_DISABLE		_IO(MC7603_IOCTL_MAGIC, 35)
/* Enable doorbell interrupt from local to PCI */
#define IOCTL_MC7603_LDB_INT_ENABLE		_IO(MC7603_IOCTL_MAGIC, 36)
/* Disable doorbell interrupt from local to PCI */
#define IOCTL_MC7603_LDB_INT_DISABLE		_IO(MC7603_IOCTL_MAGIC, 37)
/* Enable mailbox interrupt from PCI */
#define IOCTL_MC7603_MBOX_INT_ENABLE		_IO(MC7603_IOCTL_MAGIC, 38)
/* Disable mailbox interrupt from PCI */
#define IOCTL_MC7603_MBOX_INT_DISABLE		_IO(MC7603_IOCTL_MAGIC, 39)

/* Enable interrupts from 6403 chips */
#define IOCTL_MC7603_CHIP_ENABLE			_IOR(MC7603_IOCTL_MAGIC, 17, long)
/* Disable interrupts from 6403 chips */
#define IOCTL_MC7603_CHIP_DISABLE		_IOR(MC7603_IOCTL_MAGIC, 18, long)
/* Set interrupt mask */
#define IOCTL_MC7603_SET_MASK			_IOR(MC7603_IOCTL_MAGIC, 19, long)
/* Get interrupt mask */
#define IOCTL_MC7603_GET_MASK			_IOW(MC7603_IOCTL_MAGIC, 33, long)
/* Wait for interrupt from MC76.03.
After performance of this request it's nesessary receive the interrupt status
(IOCTL_MC7603_GET_INTERRUPT_STATUS). If status is 0, then timeout.
Besides this request allows the following interrupts*/
#define IOCTL_MC7603_WAIT_INTERRUPT		_IO(MC7603_IOCTL_MAGIC, 20)
/* Cancel interrupt expectation */
#define IOCTL_MC7603_CANCEL_WAIT			_IO(MC7603_IOCTL_MAGIC, 21)
/* Get interrupt status (see IOCTL_MC7603_WAIT_INTERRUPT) */
#define IOCTL_MC7603_GET_INTERRUPT_STATUS	_IOW(MC7603_IOCTL_MAGIC, 22, long)
/* Write 32-bit word to the PLX register.
Parameter of this request is the pointer to structure mc_port (see below )*/
#define IOCTL_MC7603_PORT_WRITE			_IOR(MC7603_IOCTL_MAGIC, 23, long)
/* Read 32-bit word from the PLX register.
Parameter of this request is the pointer to structure mc_port (see below )*/
#define IOCTL_MC7603_PORT_READ			_IOR(MC7603_IOCTL_MAGIC, 24, long)
/* Reset MC76.03 */
#define IOCTL_MC7603_RESET			_IO(MC7603_IOCTL_MAGIC, 25)
/* Interrupt MC76.03 */
#define IOCTL_MC7603_INTERRUPT			_IOR(MC7603_IOCTL_MAGIC, 26, long)
/* Turn on the red LED */
#define IOCTL_MC7603_LED_ON			_IO(MC7603_IOCTL_MAGIC, 27)
/* Turn off the red LED */
#define IOCTL_MC7603_LED_OFF			_IO(MC7603_IOCTL_MAGIC, 28)
/* Toggle the red LED */
#define IOCTL_MC7603_LED_TOGGLE			_IO(MC7603_IOCTL_MAGIC, 29)
/* Debug features */
#define IOCTL_MC7603_GET_INT_COUNT		_IOW(MC7603_IOCTL_MAGIC, 30, long)
#define IOCTL_MC7603_RESET_INT_COUNT		_IO(MC7603_IOCTL_MAGIC, 31)
/* RUK registers access */
#define IOCTL_MC7603_SET_RUK			_IOR(MC7603_IOCTL_MAGIC, 42, long)
#define IOCTL_MC7603_GET_RUK			_IOW(MC7603_IOCTL_MAGIC, 43, long)

#ifdef __KERNEL__
#include <linux/spinlock.h>
#include <linux/sched.h>
#include <linux/types.h>
#include <linux/cdev.h>

#define RC_MODULE_VENDOR_ID 0x6403
#define MC7603_DEVICE_ID    0x7606
#define MC7603_MAJOR        240
#define MC7603_FILENAME     "mc7603"
#define SUCCESS 0

/* Internal structure of parameters */
struct mc7603_device
{
	int		num;
	char		*name;
	int		open;		/* open flag */

	unsigned long	mem0;		/* not mapped */
	unsigned long	mem1;		/* not mapped */
	unsigned long	mmio;		/* not mapped */
	unsigned long	mmio_m;		/* mapped registers */
	unsigned long	space0;		/* mapped shared memory */
	unsigned long	space1;		/* mapped space1 */
	unsigned long	size_mmio;	/* size of mmio */
	unsigned long	size_space0;	/* size of space0 */
	unsigned long	size_space1;	/* size of space1 */

	int		irq;		/* IRQ */
	volatile int	int_reg;	/* Duplicate of the interrupt register */
	volatile unsigned long	interrupt;	/* Flag of interrupt processing */
	int		timeout;	/* Timeout */
	struct cdev	cdev;
	wait_queue_head_t queue;
#ifdef RT_MODE
	raw_spinlock_t lock;
#else
	spinlock_t	lock;		/* spinlock */
#endif
	struct semaphore sem;
	volatile int int_dma;
	struct page **pages;
//	unsigned int npages;
	struct scatterlist *sclist;
	struct plx_scatterlist *plx_sclist;
	int boot_rc;			/* result of boot test */
	int nirq;			/* irq counter */
	volatile int 	cancel_flag;
};

struct plx_scatterlist
{
	u32 pci_addr;
	u32 local_addr;
	u32 size;
	u32 next;
};

#define TYPE(dev)	(MINOR(dev) >> 4)
#define NUM(dev)	(MINOR(dev) & 0xF)
#endif /* #ifdef __KERNEL__ */

/* Structure for read/write PLX registers */
struct mc_port
{
	unsigned int addr;	/* Register offset */
	unsigned int value;	/* Value for read/write */
};

#endif /* #ifndef _MC7603_H */
