#define _GNU_SOURCE
#include <errno.h>
#include <getopt.h>
#include <libgen.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <sys/mman.h>
#include <sys/stat.h>
#include <sys/types.h>

#include <string.h>

#include "mct_test.h"

int verbose = 0;
char dev_file_path[1024];
char type_of_device[32];
int type_of_dev = COMMON;
unsigned int user_setup_params = 0x0;
int device_num = 0;
int now = -1;
int check = -1;

type_record_t types_table[] = {
	{ETHER, "ETH"},
	{HDD, "HDD"},
	{IDE, "IDE"},
	{SCSI, "SCSI"},
	{CPU, "CPU"},
	{MC404, "MC4.04"},
	{MC7603, "MC76.03"},
	{COMMON, "COMMON"},
	{MEM, "MEM"},
	{CDROM, "CDROM"},
	{-1, ""},
};

static void usage(void) {
	printf("\n");
#ifdef TEST_VER
	printf(" Version: %s\n", TEST_VER);
#else
	printf(" Version: unknown\n");
#endif
#ifdef BUILD_DATE
	printf(" Build date: %s\n", BUILD_DATE);
#else
	printf(" Build date: unknown\n");
#endif

	printf("\n\t./mct_test [-v|-t type|-d device file]\n\n");
}


static void usage_full(void) {
	int i = 0;
	usage();
	printf("\n"
	       "  Usage next args:\n"
	       "\t-v - verbose mode\n"
	       "\t-d <PATH> - path to \"/dev/...\" file\n"
	       "\t            or special name for {CPU, ETH};\n"
	       "\t-t <TYPE> - type of the device:");
	while(types_table[i].type_string[0] != '\0') {
		printf("\n\t\t- %s", types_table[i].type_string);
		i++;
	}
	printf(" - default set\n\n"
	       " Return:\n"
	       " - 0 for PASS result;\n"
	       " - 1 for FAIL result;\n\n");
}


static int set_type(char *type_of_dev) {
	int i = 0;

	while(types_table[i].type_string[0] != '\0') {
		if (strcasestr(type_of_dev, types_table[i].type_string)) {
			myprintf("ret [%d] {%s}\n",
				 i, types_table[i].type_string);
			return types_table[i].type_id;
		}
		i++;
	}
	myprintf("ret [%d] {COMMON}\n", COMMON);
	return COMMON;
}


static int parse_args(int argc, char **argv) {
	int c;

	while (1) {
		const char* short_options = "t:d:vhH?N:C:";
		const struct option long_options[] = {
			{"verbose", no_argument, 0, 'v'},
			{"", no_argument, 0, '?'},
			{"help", no_argument, 0, 'h'},
			{"Help", no_argument, 0, 'H'},
			{"type", required_argument, 0, 't'},
			{"device", required_argument, 0, 'd'},
			{"now", required_argument, 0, 'N'},
			{"check", required_argument, 0, 'C'},
			{0, 0, 0, 0}
		};

		int index;
		if ((c = getopt_long(argc, argv, short_options, long_options, &index)) == -1)
			break;

		switch (c) {
		case 'v':
			verbose = 1;
			break;
		case 'N':
			now = atoi(optarg);
			break;
		case 'C':
			check = atoi(optarg);
			break;
		case 't':
			sprintf(type_of_device, "%s", optarg);
			type_of_dev = set_type(type_of_device);
			user_setup_params |= TYPE_DEF;
			break;
		case 'd':
			sprintf(dev_file_path, "%s", optarg);
			user_setup_params |= DEV_PATH_DEF;
			break;
		case '?':
		case 'h':
			usage();
			exit(TEST_PASS);
			break;
		case 'H':
			usage_full();
			exit(TEST_PASS);
			break;
		default:
			exit(TEST_FAIL);
			break;
		}
	}

	if (!user_setup_params) {
		myprintf("Attention: Path to /dev file is undefined\n");
	}
	return 0;
}


static int run_default_test(void) {
	int fd;

	myprintf("run default test...\n");

	myprintf("open device: [%s]\n", dev_file_path);
	fd = open(dev_file_path, O_RDWR);
	if (fd == -1) {
		myprintf("Error: %s - %s\n",
			 dev_file_path, strerror(errno));
		return TEST_FAIL;
	}

	myprintf("close device: [%s]\n", dev_file_path);
	close(fd);

	return TEST_PASS;
}

extern int run_eth_test(void);
extern int run_hdd_test(void);
extern int run_cpu_test(int check);
extern int run_mem_test(int now, int check);
extern int run_mc404_test(void);
extern int run_mc7603_test(void);

static int run_test(void) {
	int res = TEST_FAIL;

	{ //debug
		int i = 0;
		while(types_table[i].type_string[0] != '\0') {
			if (types_table[i].type_id == type_of_dev)
				break;
			i++;
		}
		myprintf("Run: type: %s\n", types_table[i].type_string);
	}

	switch(type_of_dev) {
	case ETHER:
		res = run_eth_test();
		break;
	case CDROM:
	case SCSI:
	case IDE:
	case HDD:
		res = run_hdd_test();
		break;
	case CPU:
		res = run_cpu_test(check);
		break;
	case MEM:
		res = run_mem_test(now, check);
		break;
	case MC404:
		res = run_mc404_test();
		break;
	case MC7603:
		res = run_mc7603_test();
		break;
	case COMMON:
	default: res = run_default_test();
		break;
	}
	return res;
}


int main(int argc, char **argv) {
	int res;
	parse_args(argc, argv);
	res = run_test();
	myprintf("result of test is: %s (%d)\n",
		 (res == 0) ? "PASS" : "FAIL",
		 res);

	return res;
}
